/**
 * myChart 1.1
 */
define(['lib/zepto.js'], function($) {
  var winRatio = 0;
  var startEvent = 'touchstart';
  function Chart(ele){
    this.ele = ele;
    this.box = {};
    this.setting = {
      type : undefined, //type, required
      tooltip : true, //has tooltip
      tooltipTrigger: false,//show the tooltip when has it
      tooltipMaker: undefined,//tooltip html maker function with parameters 'dot' and 'svg dom'
      callback: undefined,//the callback function of click on item
      moveTrigger: true,//trigger callback when move in axis
      defaultEvent: true//whether use defaute events on item
    }
  }
  function setChart(ele,option){
    try{
      var $dom = $(ele);      
      var obj = new Chart(ele);
      var originalBox = $dom.attr('viewBox');
      if(typeof originalBox === 'string'){
        var oArr = originalBox.split(',');
        obj.box.width = oArr[2];
        obj.box.height = oArr[3];
      }else{
        obj.box.width = $dom.attr('width');
        obj.box.height = $dom.attr('height');
      }
      $.extend(true, obj.setting, option);
      if(!obj.setting.type){
        console.error('Chart type can\'t be undefined');
        return false;
      }
      if(obj.setting.type === 'bar'){
        eventBindBar(obj);
      }
      if(obj.setting.type === 'pie'){
        eventBindPie(obj);
      }
      if(obj.setting.type === 'line'){
        eventBindLine(obj);
      }
      if(obj.setting.type === 'area' || obj.setting.type === 'scatter'){
        eventBindOther(obj);
      }
      
      if(obj.setting.tooltip && obj.setting.tooltipTrigger && $dom.find('.dot.selected').length){
        var idx = $dom.find('.group-real .dot').length;
        $dom.find('.inner-line-mask .tick:nth-of-type('+(idx)+')').triggerHandler(startEvent);
      }  
    }catch(e){
      console.error(e);
    }   
  }
  function eventBindBar(obj){
    var _this = obj;
    var $dom = $(_this.ele);
    $dom.find('.bar').on('click',function(e){
      if(_this.setting.callback && (typeof _this.setting.callback === 'function')){
        _this.setting.callback();
      }
    });
  }

  function eventBindPie(obj){
    var _this = obj;
    var $dom = $(_this.ele);
    var startEvent = 'ontouchmove' in document ? 'touchstart' : 'click';
    $dom.find('.pie-item').on(startEvent,function(e){
      var idx = ($(this).parents('svg').find('.pie-item')).index($(this));
      var info = $(this).data('info').split(';');
      var $parent = $(this).parents('.svg-pie');
      $parent.find('.active').attr('class', 'pie-bg');
      $parent.find('.pie-bg').eq(idx).attr('class', 'pie-bg active');
      $parent.find('.pie-label-name').text(info[1]);
      $parent.find('.pie-label-num').css('color', info[2]).text(info[0]);
      if(_this.setting.callback && (typeof _this.setting.callback === 'function')){
        _this.setting.callback();
      }
    });
  }

  function eventBindLine(obj){
    var _this = obj;
    var $dom = $(_this.ele);
    winRatio = $dom.parent().width()/obj.box.width;
    calHeight = winRatio*obj.box.height;
    var ticks = $dom.find(".inner-line-mask .tick");
    var tickTotalWidth = winRatio*(obj.box.width-40);
    var tickWidth = tickTotalWidth/ticks.length;
    var touchOrigin = { x:0, y:0};
    $dom.find('.inner-line-mask .tick>line').css('stroke-width', tickWidth);
    startEvent = 'ontouchmove' in document ? 'touchstart' : 'mousedown';
    $dom.parent().find('.legend-cell-d').on(startEvent,function(){
      var idx = $(this).index();
      $(this).toggleClass('vain');
      $dom.find('.line').eq(idx).toggleClass('hide');
      $dom.find('.dot-group').eq(idx).toggleClass('hide');
      $dom.parent().find('.tooltip-d').each(function(){
        if($(this).data('idx') == idx){
          $(this).toggle();
        }
      });
    }); 
    $('*').on(startEvent, function(e){
      if($(e.target).parents('.my-chart').length === 0){
        $dom.siblings(".tooltip").html('').css('display','none');
        $dom.find(".tick.selected").removeClass('selected');
        $dom.find(".dot.selected").removeClass('selected');
      }
    });
    $dom.find('.inner-line-mask .tick').on(startEvent, function(e){
      if(_this.setting.defaultEvent){
        var i = $(this).index();
        if($dom.find(".tick.selected").length > 0){
          $dom.find(".tick.selected").removeClass('selected');
          $dom.find(".dot.selected").removeClass('selected');
        }

        $dom.find('.inner-line-x .tick:nth-child('+(i+1)+')').addClass('selected');
        $dom.find('.dot-group .dot:nth-of-type('+(i)+')').addClass('selected');
      } 
      if(startEvent === 'touchstart'){
        if(e.changedTouches){
          touchOrigin.x = e.changedTouches[0].clientX;
          touchOrigin.y = e.changedTouches[0].clientY;
        }      
      }       
      if(_this.setting.tooltip){
        setMultiTooltip(_this, winRatio);
      }     
      if(_this.setting.callback && (typeof _this.setting.callback === 'function')){
        _this.setting.callback();
      }
    });
    
    if(_this.setting.moveTrigger){
      if(startEvent === 'touchstart'){
        $dom.on('touchmove', function(e){
          var movePosition = {};
          movePosition.x = e.changedTouches[0].clientX;
          movePosition.y = e.changedTouches[0].clientY;
          if(touchDirection(touchOrigin, movePosition) === 'horizon'){
            $(document.elementFromPoint(movePosition.x, movePosition.y)).parent().triggerHandler('touchstart');
            return false;
          } 
        });
      }else{
        $dom.find('.inner-line-mask .tick').on('mousedown', function(e){
          $dom.on('mousemove', function(e){
            $(document.elementFromPoint(e.clientX, e.clientY)).parent().triggerHandler('mousedown');
          });
        });
        $dom.find('.inner-line-mask .tick').on('mouseup', function(e){
          $dom.off('mousemove');   
        });
      }   
    }   
  }

  function eventBindOther(obj){
    var _this = obj;
    var $dom = $(_this.ele);
    winRatio = $dom.parent().width()/obj.box.width;
    calHeight = winRatio*obj.box.height;
    var ticks = $dom.find(".inner-line-mask .tick");
    var tickTotalWidth = winRatio*(obj.box.width-40);
    var tickWidth = tickTotalWidth/ticks.length;
    var realGroup = $dom.find(".group-real .dot");
    var touchOrigin = { x:0, y:0};
    $dom.find('.inner-line-mask .tick>line').css('stroke-width', tickWidth);
    startEvent = 'ontouchmove' in document ? 'touchstart' : 'mousedown';
    $dom.find('.inner-line-mask .tick').on(startEvent, function(e){
      if(_this.setting.defaultEvent){
        var i = $(this).index();
        if($dom.find(".tick.selected").length > 0){
          $dom.find(".tick.selected").removeClass('selected');
          $dom.find(".dot.selected").removeClass('selected');
        }

        $dom.find('.inner-line-x .tick:nth-child('+(i+1)+')').addClass('selected');
        if(realGroup.length === 0 || (i > realGroup.length)){
          $dom.find('.group-forecast .dot:nth-of-type('+(i-(realGroup.length?(realGroup.length-1):(0)))+')').addClass('selected');
        }else{
          if(i === realGroup.length && $dom.find('.group-forecast .dot').length){
            $dom.find('.group-forecast .dot:first-child').addClass('selected');
          }else{
            $dom.find('.group-real .dot:nth-of-type('+(i)+')').addClass('selected');
          }
        }
      }
      if(startEvent === 'touchstart'){
        if(e.changedTouches){
          touchOrigin.x = e.changedTouches[0].clientX;
          touchOrigin.y = e.changedTouches[0].clientY;
        }      
      }     
      if(_this.setting.tooltip){
        setTooltip(_this.ele, winRatio);
      }     
      if(_this.setting.callback && (typeof _this.setting.callback === 'function')){
        _this.setting.callback();
      }
    });
    
    if(_this.setting.moveTrigger){
      if(startEvent === 'touchstart'){
        $dom.on('touchmove', function(e){
          var movePosition = {};
          movePosition.x = e.changedTouches[0].clientX;
          movePosition.y = e.changedTouches[0].clientY;
          if(touchDirection(touchOrigin, movePosition) === 'horizon'){
            $(document.elementFromPoint(movePosition.x, movePosition.y)).parent().triggerHandler('touchstart');
            return false;
          }          
        });
      }else{
        $dom.find('.inner-line-mask .tick').on('mousedown', function(e){
          $dom.on('mousemove', function(e){
            $(document.elementFromPoint(e.clientX, e.clientY)).parent().triggerHandler('mousedown');
          });
        });
        $dom.find('.inner-line-mask .tick').on('mouseup', function(e){
          $dom.off('mousemove');   
        });
      }   
    }   
  }
  function setTooltip(ele){
    var $dom = $(ele);
    var toolTip = $dom.siblings(".tooltip").get(0);
    var toolRange = [20, $dom.parent().width() - 10];
    var dotSel = $dom.find(".dot.selected").get(0);
    if(dotSel){
      toolTip.innerHTML = dotSel.getAttribute('data-info');
      calShift(dotSel, toolTip, toolRange);
    } 
  }
  function setMultiTooltip(obj){
    var $dom = $(obj.ele);
    $dom.siblings(".tooltip").css('display','unset');
    var toolTip = $dom.siblings(".tooltip").get(0);
    var toolRange = [20, $dom.parent().width() - 10];
    var dotSel = $dom.find(".dot.selected");
    if(dotSel.length > 0){
      toolTip.innerHTML = obj.setting.tooltipMaker?_this.setting.tooltipMaker(dotSel, $dom):getTooltipCon(dotSel, $dom);
      calSort(toolTip);
      calShift(dotSel.get(0), toolTip, toolRange);
    } 
  }
  function calShift(d,t,r){
    var tWidth = t.offsetWidth;
    var dShift = (50.5+1*d.getAttribute('cx'))*winRatio;
    
    var tShift = dShift - tWidth/2;
    if(tShift < r[0]){
      tShift = r[0];
    }
    if(tShift + tWidth > r[1]){
      tShift = r[1] - tWidth;
    }
    t.style.left = tShift + 'px';
  }
  function getTooltipCon(d, $dom){
    var ret = '';
    var lineTypeArr = [];
    var lineStatusArr = [];
    $dom.find('.line').each(function(i,n){
      var clazz = $(n).attr('class').split(' ');
      lineTypeArr.push(clazz[1]);
      if($(n).hasClass('hide')){
        lineStatusArr.push(i);
      }
    });
    d.each(function(i,n){
      var info = $(n).data('info').split(';');
      if(i === 0){
        ret += '<div class="tooltip-date">' + info[0] + '</div>';
        if(info[4]){
          ret += '<div class="tooltip-label">' + info[4] + '</div>';
        }
        ret += '<div class="tooltip-info">';
      }
      var style = '';
      if($.inArray(i, lineStatusArr) >= 0){
        style = 'style="display: none;"';
      }
      if($dom.parent().attr('color-type') === 'colorize'){
        ret += '<div '+style+' class="tooltip-d" data-count="'+info[3]+'" data-idx="'+i+'">' + info[1] + ' : ' + '<span style="color:'+ lineTypeArr[i] +'"> ' + info[2] + '</span></div>';
      }else{
        ret += '<div '+style+' class="tooltip-'+ lineTypeArr[i] +'" data-count="'+info[3]+'" data-idx="'+i+'">' + info[1] + ' : ' + '<span> ' + info[2] + '</span></div>';
      }
      ret += (i===(d.length-1))?'</div>':'';
    });
    return ret;
  }
  function calSort(t){
    var h = $(t).find('.tooltip-d').sort(function(a, b){
      var contentA =parseInt( $(a).attr('data-count'));
      var contentB =parseInt( $(b).attr('data-count'));
      contentA = isNaN(contentA)?0:contentA;
      contentB = isNaN(contentB)?0:contentB;
      return (contentA > contentB) ? -1 : (contentA < contentB) ? 1 : 0;
    });
    $(t).find('.tooltip-info').html(h);
  }
  function touchDirection(origin, now){
    var ret = 'horizon';
    if(origin.x === 0 && origin.y === 0){
      return ret;
    }else{
      var dx = Math.abs(now.x - origin.x);
      var dy = Math.abs(now.y - origin.y);
      if(dy > dx){
        ret = 'vertical';
      }
    }
    return ret;
  }
  
  return setChart;
});
