'use strict';

var d3 = require('d3');

var calxAxis = function calxAxis(n) {
  var num = 'even';
  if (n < 8) {
    num = 'n';
  }
  if (n > 17) {
    if (n % 3 === 0) {
      num = '3n+1';
    } else {
      num = '4n+1';
    }
  }
  if (n > 30) {
    num = '5n';
  }
  if (n > 50) {
    num = Math.floor(n * 0.12) + 'n';
  }
  return num;
};

var stat = function stat(data, scales, id, box, type) {
  var type = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 0;
  var ticks = 4;

  var xAxis = d3.axisBottom(scales.xAxis).ticks(ticks);
  if (data.type === 'category') {
    xAxis = d3.axisBottom(scales.xAxis).ticks(ticks);
  }
  if (data.type === 'time') {
    xAxis = d3.axisBottom(scales.xAxis).tickFormat(d3.timeFormat('%-m/%-d'));
  }
  if (type === 1) {
    xAxis = d3.axisBottom(scales.xAxisPlus).tickValues(scales.xValues);
  }
  
  var yAxis = d3.axisLeft(scales.yAxis).ticks(ticks);
  if (scales.yUnit) {
    var unitFormat = function unitFormat(d) {
      return d + scales.yUnit;
    };
    yAxis.tickFormat(unitFormat);
  }else{
    if (scales.yAxisRange && scales.yAxisRange[1] >= 230000) {
      var unitFormat = function unitFormat(d) {
        return (d >= 10000) ? `${ parseInt(d * 0.0001) }万` : d;
      };
      yAxis.tickFormat(unitFormat);
    }   
  }

  d3.select('#' + id).append('g').attr('class', 'x axis').attr('transform', 'translate(' + (box.left) + ',' + (box.height - box.bottom - 0.5) + ')').call(xAxis);
  d3.select('#' + id).append('g').attr('class', 'y axis').attr('transform', 'translate(' + (box.left) + ',' + box.top + ')').call(yAxis);

  var yInner = d3.axisLeft(scales.yAxis).ticks(ticks).tickSize(box.left + box.right  - box.width).tickFormat('');
  d3.select('#' + id).append('g').attr('class', 'inner-line-y').attr('transform', 'translate(' + (box.left) + ',' + box.top + ')').call(yInner);

  var xInner = d3.axisBottom(scales.xAxis).ticks(ticks).tickSize(box.bottom + box.top - box.height).tickFormat('');
  d3.select('#' + id).append('g').attr('class', 'inner-line-x').attr('transform', 'translate(' + box.left + ',' + (box.height - box.bottom) + ')').call(xInner);
  d3.select('#' + id).selectAll('.axis.x text').attr('opacity', 0);

  var sel = d3.select('#' + id).selectAll('.x .tick:nth-child(' + calxAxis(data.data.length) + ')');
  sel.select('text').attr('opacity', 1);
};

module.exports = stat;