'use strict';

var d3 = require('d3');
var tool = require('./tool.js');

var XMLSerializer = require('xmldom-qsa').XMLSerializer;

var svgWrap = void 0;
var scales = {};
var box = {};
var setting = {};
/* 绘制坐标系 */
var drawAxis = function drawAxis(data, id) {
  svgWrap.attr('height', box.height).attr('width', box.width);
  svgWrap.attr('id', id).attr('viewBox', '0,0,'+box.width+','+box.height)
  .attr('preserveAspectRatio', 'xMidYMid meet');
  svgWrap.html('');
  var _data = data.series[0].data;
  scales.xAxis = tool.axisHandle(data.xAxis,[0, box.width - box.left - box.right]);
  scales.xAxisPlus = tool.axisHandle(data.xAxis,[30, box.width - box.left - box.right - 30]);
  scales.xValues = data.xAxis.data;
  scales.yAxis = tool.axisHandle(data.yAxis,[0, box.height - box.bottom - box.top],scales, true);
  scales.yUnit = data.yAxis.unit || '';
  tool.axisGener(data.xAxis, scales, id, box, 1);
  return _data;
};
/* 绘制面积图 */
var drawArea = function drawArea(svgObj, data) {
  tool.barGener(svgObj, data, scales, box);
};

var d3g = {
  init: function init(data, config) {
    setting = {};
    box = {};
    tool.extend(box, tool.box);
    var defaults = {
      id : 'bar_'+(new Date()).getTime(),
      debug: false
    }
    tool.extend(setting, defaults);
    if(config){
      if(typeof config === 'string'){
        setting.id = config;
      }else{
        tool.extend(setting, config);
        if(config.box){
          tool.extend(box, config.box);
        }
      }
    }else{
      setting = defaults;
    }
    try {
      if (!data) {
        setting.debug && console.log('no bar data');
        return false;
      }
      setting.debug && console.log('==== Bar Chart '+setting.id+' start -->');
      setting.debug && console.time('==== Bar Chart '+setting.id);
      var id = setting.id || ('bar_'+(new Date()).getTime());
      document.querySelector('.svg-wrap').removeChild('*');
      d3.select('.svg-wrap').append('div').attr('class', 'svg-bar my-chart');
      svgWrap = d3.select(document.querySelector('.svg-bar')).append('svg');
      var _data = drawAxis(data, id);
      drawArea(id, _data);
      svgWrap.attr('width','100%')
      .attr('height','100%');
      var s = new XMLSerializer();
      var d = document.querySelector('.svg-bar');
      var str = s.serializeToString(d);
      setting.debug && console.timeEnd('==== Bar Chart '+setting.id);
      return str;
    } catch(e) {
      setting.debug && console.log(e);
      return false;
    }
  }
};
module.exports = d3g.init;