'use strict';

var d3 = require('d3');
var tool = require('./tool.js');
var XMLSerializer = require('xmldom-qsa').XMLSerializer;
var DOMParser = require('xmldom-qsa').DOMParser;
var lineType = 'single';
var svgWrap = void 0;
var scales = { colorType : 'index'};
var box = {};
var idx = 0;
var colorReg = /^(#[a-f0-9]{6}|#[a-f0-9]{3}|rgb *\( *[0-9]{1,3}%? *, *[0-9]{1,3}%? *, *[0-9]{1,3}%? *\)|rgba *\( *[0-9]{1,3}%? *, *[0-9]{1,3}%? *, *[0-9]{1,3}%? *, *[0-9]{1,3}%? *\)|black|green|silver|gray|olive|white|yellow|maroon|navy|red|blue|purple|teal|fuchsia|aqua)$/i;
var setting = {};
/* 绘制坐标系 */
var drawAxis = function drawAxis(data, id) {
  svgWrap.attr('height', box.height).attr('width', box.width);
  svgWrap.attr('id', id).attr('viewBox', '0,0,'+box.width+','+box.height)
  .attr('preserveAspectRatio', 'xMidYMid meet');
  var _data = data;
  scales.xSelected = data.tooltip.pointIndex;
  scales.xAxis = tool.axisHandle(data.xAxis,[0, box.width - box.left - box.right]);
  scales.yAxis = tool.axisHandle(data.yAxis,[box.height - box.bottom - box.top, 0],scales,true);
  setting.debug && console.time('- d3 draw axis');
  tool.axisGener(_data.xAxis, scales, id, box);
  setting.debug && console.timeEnd('- d3 draw axis');
  return _data;
};
/* 绘制面积图 */
var drawArea = function drawArea(svgObj, param, data) { 
  setting.debug && console.time('- d3 draw line');
  if(lineType === 'single'){
    tool.areaGener(svgObj, param, scales, data.data, box);
  }  
  var line = tool.lineGener(svgObj, param, scales, data.data, box);
  setting.debug && console.timeEnd('- d3 draw line');
  var dotGroupClass = 'dot-group group-' + ((scales.colorType === 'index')?param:('color-'+(idx++) + ' no-color'));
  var dotClass = function dotClass(d) {
    var ret = 'dot';
    if (d.label) {
      ret = 'dot mark';
    }
    return ret;
  };
  var dotInfo = function dotInfo(d) {
    return d.tooltip + ((d.label===true)?'':(d.label || ''));
  };
  var dotFill = function dotFill(d) {
    var ret = param;
    return ret;
  }
  setting.debug && console.time('- d3 draw dots');
  tool.dotGener(box, svgObj, data.data, dotGroupClass, dotClass, dotInfo, dotFill, line, param,scales);
  setting.debug && console.timeEnd('- d3 draw dots');
};

var drawTooltip = function drawTooltip(dot) {
  var dotNode = new DOMParser().parseFromString('<div class="tooltip">' + dot.attr('data-info') + '</div>');
  document.querySelector('.svg-line').appendChild(dotNode);
};

var drawLegend = function drawLegend(names, classes){
  if(names.length !== classes.length) return false;
  var html = '<div class="legend">';
  for(var i = 0; i<names.length; i++){
    if(scales.colorType === 'index'){
      html += '<div class="legend-cell-'+classes[i]+'"><b>•</b><span>'+names[i]+'</span></div>';
    }else{
      html += '<div class="legend-cell-d" style="color:' +classes[i]+'"><b style="background-color:' +classes[i]+'">•</b><span>'+names[i]+'</span></div>';
    }
  }
  html += '</div>';
  var dotNode = new DOMParser().parseFromString(html);
  document.querySelector('.svg-line').appendChild(dotNode);
}
var init = function init(data, config, type) {
  setting = {};
  box = {};
  tool.extend(box, tool.box);
  var defaults = {
    id : 'line_'+(new Date()).getTime(),
    debug: false
  }
  tool.extend(setting, defaults);
  if(config){
    if(typeof config === 'string'){
      setting.id = config;
    }else{
      tool.extend(setting, config);
      if(config.box){
        tool.extend(box, config.box);
      }
    }
  }else{
    setting = defaults;
  }
  try {
    if (data.xAxis.data.length <= 1) {
      setting.debug && console.log('no line data');
      return false;
    }
    setting.debug && console.log('==== Line Chart '+setting.id+' start -->');
    setting.debug && console.time('==== Line Chart '+setting.id);
    lineType = type;
    scales.colorType = 'index';
    var id = setting.id || ('line_'+(new Date()).getTime());
    document.querySelector('.svg-wrap').removeChild('*');
    d3.select('.svg-wrap').append('div').attr('class', 'svg-line my-chart');
    d3.select('.svg-line').attr('class', 'svg-line my-chart ' + lineType);
    d3.select('.svg-line').attr('color-type', 'index');
    svgWrap = d3.select(document.querySelector('.svg-line')).append('svg');
    setting.debug && console.time('all axis');
    var _data = drawAxis(data, id);
    setting.debug && console.timeEnd('all axis');
    var colors = ['#ff574d', '#3e8ff4', '#2bc7ff', '#20d8dc', '#35cd6a', '#88dd2c',
    '#ffef00', '#ffd000', '#fbaa00', '#fa7900', '#d02e0e', '#7e5cff'];
    var seriesArr = (lineType === 'single') ? ['real', 'forecast'] : colors;
    var sL = _data.series.length;
    var seriesNameArr = [];
    var color = [];
    var isColor = function (value) {
      if(colorReg.test(value)) return true;
      return false;
    }
    setting.debug && console.time('all line');
    for(var i = 0; i < sL; i++){     
      if(lineType !== 'single'){
        if (isColor(_data.series[i].color)) {
          color[i] = _data.series[i].color;          
        } else {
          color[i] = seriesArr[i];
        }
        scales.colorType = 'colorize';
        d3.select('.svg-line').attr('color-type', 'colorize');        
      }else{
        color[i] = seriesArr[i];
      }
      drawArea(id, color[i],  _data.series[i]);
      if(_data.series[i].name){
        seriesNameArr.push(_data.series[i].name);
      }
    }
    tool.maskGener(id, scales, box);
    if(seriesNameArr.length > 0){
      drawLegend(seriesNameArr, color);
    }
    setting.debug && console.timeEnd('all line');
    if (lineType === 'single' && scales.xSelected >= 0 && scales.xSelected !== 999.9) {
      var dot = d3.select('.group-forecast .dot:first-child').attr('class', 'dot selected');
      drawTooltip(dot);
    } else {
      var dotNode = new DOMParser().parseFromString('<div class="tooltip"> </div>');
      document.querySelector('.svg-line').appendChild(dotNode);
    }
    svgWrap.attr('width','100%')
    .attr('height','100%');
    var s = new XMLSerializer();
    var d = document.querySelector('.svg-line');
    var str = s.serializeToString(d);
    setting.debug && console.timeEnd('==== Line Chart '+setting.id);
    return str;
  } catch(e) {
    setting.debug && console.log(e);
    return false;
  }
};
var d3g = {
  line : function(data, config){
    return init(data, config, 'multi');
  },
  area : function(data, config){
    return init(data, config, 'single');
  }  
};
module.exports = d3g;