'use strict';

var d3 = require('d3');
var tool = require('./tool.js');
var XMLSerializer = require('xmldom-qsa').XMLSerializer;
var DOMParser = require('xmldom-qsa').DOMParser;

var svgWrap = void 0;

var box = {
  height: 250,
  width: 320,
  left: 30,
  bottom: 18
};
var setting = {};

var colors = ['#ff574d', '#3e8ff4', '#2bc7ff', '#20d8dc', '#35cd6a', '#88dd2c',
    '#ffef00', '#ffd000', '#fbaa00', '#fa7900', '#d02e0e', '#7e5cff'];

var getPosition = function getPosition(pos) {
  var d = 20;
  var p = { x: 1, y: 1 };
  var ratio = Math.abs(pos.x) / Math.abs(pos.y);
  var x = void 0;
  var y = void 0;
  var tx = 0.5;
  if (pos.x < 0) {
    p.x = -1;
    tx = 64;
  }
  if (pos.y < 0) {
    p.y = -1;
  }
  if (ratio >= 1) {
    x = pos.x * 1 + d * p.x;
    y = x * pos.y / pos.x;
  } else {
    y = pos.y * 1 + d * p.y;
    x = pos.x * y / pos.y;
  }
  return { line: [{ x: x, y: y }, { x: x + p.x * 8.5, y: y }], text: { x: x + p.x * (8.5 + tx), y: y + 4 } };
};

/* 绘制 */
var drawPie = function drawPie(data, id) {
  svgWrap.attr('height', box.height).attr('width', box.width);
  svgWrap.attr('id', id);
  svgWrap.html('');
  var _data = data;
  var color = _data.color || colors;
  var arc = d3.arc().innerRadius(50).outerRadius(80);
  var pie = d3.pie();

  var arcbg = d3.arc().innerRadius(50).outerRadius(86);
  svgWrap.selectAll('path.pie-bg').data(pie(_data.value)).enter().append('path').attr('fill', function (d, i) {
    return color[i];
  }).attr('class', function (d, i) {
    return i === 0 ? 'pie-bg active' : 'pie-bg';
  }).attr('d', arcbg).attr('transform', 'translate(160,125)');

  svgWrap.selectAll('path.pie-item').data(pie(_data.value)).enter().append('path').attr('class', 'pie-item').attr('fill', function (d, i) {
    return color[i];
  }).attr('d', arc).attr('transform', 'translate(160,125)').attr('data-info', function (d, i) {
    return _data.value[i] + (_data.unit || '') + ';' + _data.name[i] + ';' + color[i] + ';' + arc.centroid(d);
  });
  var labelItem = document.querySelectorAll('.pie-item');
  var labelSize = labelItem.length <= 5 ? labelItem.length : 5;
  var labelLine = [];
  var lineFunc = d3.line().x(function (d) {
    return d.x;
  }).y(function (d) {
    return d.y;
  });
  svgWrap.append('g').attr('class', 'pie-label-line').attr('transform', 'translate(160,125)');
  for (var i = 0; i < labelSize; i++) {
    var color = labelItem[i].getAttribute('data-info').split(';')[2];
    var text = labelItem[i].getAttribute('data-info').split(';')[1] + ' ' + labelItem[i].getAttribute('data-info').split(';')[0];
    var pos = labelItem[i].getAttribute('data-info').split(';')[3].split(',');
    labelLine[i] = [];
    labelLine[i].push({ x: pos[0], y: pos[1] });
    var posGroup = getPosition({ x: pos[0], y: pos[1] });
    labelLine[i] = labelLine[i].concat(posGroup.line);
    svgWrap.select('.pie-label-line').append('path').attr('d', lineFunc(labelLine[i])).attr('stroke', color).attr('stroke-width', 1).attr('fill', 'none');
    svgWrap.select('.pie-label-line').append('text').attr('x', posGroup.text.x).attr('y', posGroup.text.y).attr('fill', color).text(text);
  }
  var nodeData = svgWrap.select('.pie-item').attr('data-info').split(';');
  var dotNode = new DOMParser().parseFromString('<div class="piechart-center-label"><div class="pie-label"><span class="pie-label-num" style="color: ' + nodeData[2] + '">' + nodeData[0] + '</span><br/><span class="pie-label-name">' + nodeData[1] + '</span></div></div>');
  document.querySelector('.svg-pie').insertBefore(dotNode, document.querySelector('#' + id));
  return _data;
};

var d3g = {
  init: function init(data, config) {
    setting = {};
    var defaults = {
      id : 'pie_'+(new Date()).getTime(),
      debug: false
    }
    tool.extend(setting, defaults);
    if(config){
      if(typeof config === 'string'){
        setting.id = config;
      }else{
        tool.extend(setting, config);
      }
    }else{
      setting = defaults;
    }
    try {
      if (!data) {
        setting.debug && console.log('no pie data');
        return false;
      }
      setting.debug && console.log('==== Pie Chart '+setting.id+' start -->');
      setting.debug && console.time('==== Pie Chart '+setting.id);
      var id = setting.id || ('pie_'+(new Date()).getTime());
      document.querySelector('.svg-wrap').removeChild('*');
      d3.select('.svg-wrap').append('div').attr('class', 'svg-pie my-chart');
      svgWrap = d3.select(document.querySelector('.svg-pie')).append('svg');
      drawPie(data, id);
      var s = new XMLSerializer();
      var d = document.querySelector('.svg-pie');
      var str = s.serializeToString(d);
      setting.debug && console.timeEnd('==== Pie Chart '+setting.id);
      return str;
    } catch(e) {
      setting.debug && console.log(e);
      return false;
    }
  }
};
module.exports = d3g.init;