'use strict';

var d3 = require('d3');
var tool = require('./tool.js');

var XMLSerializer = require('xmldom-qsa').XMLSerializer;
var DOMParser = require('xmldom-qsa').DOMParser;

var svgWrap = void 0;
var scales = { colorType : 'index'};
var box = {};
var setting = {};

/* 绘制坐标系 */
var drawAxis = function drawAxis(data, id) {
  svgWrap.attr('height', box.height).attr('width', box.width);
  svgWrap.attr('id', id).attr('viewBox', '0,0,'+box.width+','+box.height)
  .attr('preserveAspectRatio', 'xMidYMid meet');
  
  svgWrap.html('');
  scales.xAxis = tool.axisHandle(data.xAxis,[0, box.width - box.left - box.right]);
  scales.yAxis = tool.axisHandle(data.yAxis,[box.height - box.bottom - box.top, 0],scales, true);
  tool.axisGener(data.xAxis, scales, id, box);
};
/* 绘制面积图 */
var drawArea = function drawArea(svgObj, param,  data) {
  tool.areaGener(svgObj, param, scales, data.series[0].data, box);
  var line = tool.lineGener(svgObj, param, scales, data.series[0].data, box);

  tool.maskGener(svgObj, scales, box);

  var dotGroupClass = 'dot-group scatter group-' + param;
  var dotClass = function dotClass(d) {
    var ret = 'dot';
    if (d.label) {
      ret = 'dot mark';
    }
    return ret;
  };
  var dotInfo = function dotInfo(d) {
    return d.tooltip + ((d.label===true)?'':d.label);
  };
  var dotFill = '';
  tool.dotGener(box, svgObj, data.series[0].data, dotGroupClass, dotClass, dotInfo, dotFill, line,param, scales);
};

var d3g = {
  init: function init(data, config) {
    setting = {};
    box = {};
    tool.extend(box, tool.box);
    var defaults = {
      id : 'scatter_'+(new Date()).getTime(),
      debug: false
    }
    tool.extend(setting, defaults);
    if(config){
      if(typeof config === 'string'){
        setting.id = config;
      }else{
        tool.extend(setting, config);
        if(config.box){
          tool.extend(box, config.box);
        }
      }
    }else{
      setting = defaults;
    }
    try {
      if (!data) {
        setting.debug && console.log('no scatter data');
        return false;
      }
      setting.debug && console.log('==== Scatter Chart '+setting.id+' start -->');
      setting.debug && console.time('==== Scatter Chart '+setting.id);
      var id = setting.id || ('scatter_'+(new Date()).getTime());
      document.querySelector('.svg-wrap').removeChild('*');
      d3.select('.svg-wrap').append('div').attr('class', 'svg-scatter my-chart');
      svgWrap = d3.select(document.querySelector('.svg-scatter')).append('svg');
      drawAxis(data, id);
      drawArea(id, 'real', data);
      var dotNode = new DOMParser().parseFromString('<div class="tooltip"> </div>');
      document.querySelector('.svg-scatter').appendChild(dotNode);
      svgWrap.attr('width','100%')
      .attr('height','100%');
      var s = new XMLSerializer();
      var d = document.querySelector('.svg-scatter');
      var str = s.serializeToString(d);
      setting.debug && console.timeEnd('==== Scatter Chart '+setting.id);
      return str;
    } catch(e) {
      setting.debug && console.log(e);
      return false;
    }
  }
};
module.exports = d3g.init;