'use strict';

var d3 = require('d3');
var axis = require('./axis.js');

var box = {
  height: 170,
  width: 360,
  left: 50,
  bottom: 18,
  right:14,
  top:12
};

var axisGener = function axisGener(_data, scales, id, box, type) {
  axis(_data, scales, id, box, type);
};

var lineGener = function lineGener(svgObj, param, scales, data, box) {
  var line = d3.line().x(function (d) {
    return scales.xAxis(d.xValue);
  }).y(function (d) {
    return !isNaN(d.yValue)?(scales.yAxis((d.yValue===0)?'0':d.yValue)):undefined;
  }).defined(function (d) {
    return !isNaN(d.yValue);
  });
  d3.select('#' + svgObj).append('path').datum(data).attr('class', 'line ' + param).attr('d', line).attr('transform', 'translate(' + (box.left + 0.5) + ',' + box.top + ')').attr('stroke',(scales.colorType === 'index'?'':param));
  return line;
};

var areaGener = function areaGener(svgObj, param, scales, data, box) {
  var area = d3.area().x(function (d) {
    return scales.xAxis(d.xValue);
  }).y1(function (d) {
    return !isNaN(d.yValue)?(scales.yAxis((d.yValue===0)?'0':d.yValue)):undefined;
  }).y0(scales.yAxis(0))
  .defined(function (d) {
    return !isNaN(d.yValue);
  });
  d3.select('#' + svgObj).append('path').datum(data).attr('class', param + ' area').attr('d', area).attr('transform', 'translate(' + (box.left + 0.5) + ',' + box.top + ')');
  return area;
};

var maskGener = function maskGener(svgObj, scales, box) {
  var xInner = d3.axisBottom(scales.xAxis).ticks(4).tickSize(box.top + box.bottom - box.height).tickFormat('');
  d3.select('#' + svgObj).append('g').attr('class', 'inner-line-mask').attr('transform', 'translate(' + box.left + ',' + (box.height - box.bottom + 0.5) + ')').call(xInner);
};

var dotGener = function dotGener(box, svgObj, data, dotGroupClass, dotClass, dotInfo, dotFill, line, param, scales) {
  d3.select('#' + svgObj).append('g').attr('class', dotGroupClass).selectAll('.dot').data(data.filter(function (d) {
    return d.yValue || true;
  })).enter().append('circle').attr('class', dotClass).attr('cx', line.x()).attr('cy', line.y()).attr('transform', 'translate(' + (box.left + 0.5) + ',' + box.top + ')').attr('data-info', dotInfo).attr('fill', dotFill).attr('stroke',(scales.colorType === 'index'?'':param));
};

var barGener = function barGener(svgObj, data, scales, box) {
  d3.select('#' + svgObj).append('g').attr('class', 'bar-group').selectAll('.bar').data(data).enter().append('g').attr('class', 'single-bar').append('rect').attr('class', 'bar').attr('x', function (d) {
    return scales.xAxisPlus(d.xValue);
  }).attr('y', function (d) {
    return scales.yAxis(d.yValue);
  }).attr('width', 28).attr('height', function (d) {
    return box.height - box.bottom - scales.yAxis(d.yValue);
  }).attr('transform', 'translate(' + (37) + ',0)')
  .attr('data-info',function(d){
    return d.xValue + ';' + d.yValue + scales.yUnit;
  });
  d3.select('#' + svgObj).selectAll('.single-bar').append('text').attr('x', function (d) {
    return scales.xAxisPlus(d.xValue);
  }).attr('y', function (d) {
    return scales.yAxis(d.yValue);
  }).text(function (d) {
    return d.yValue + scales.yUnit;
  }).attr('fill', '#8f9296').attr('transform', function (d) {
    return 'translate(' + (37 - ((d.yValue + scales.yUnit).replace('.', '').length * 7.5 - 28) * 0.5) + ',-4)';
  });
};

function domainHandle(d) {
  if(d[1] <= 1){
    return [0, 1];
  }
  var max = d[1].toExponential(2);
  var maxScale = max.split('e+');
  if (maxScale[0] < 2) {
    maxScale[0] = Math.ceil(maxScale[0] * 10);
    if (maxScale[0] <= 15 && (maxScale[0] % 3 !== 0)) {
      maxScale[0] = 3 * Math.floor(maxScale[0] / 3) + 3;
    }
    if(maxScale[0] === 9) maxScale[0]++;
    if (maxScale[0] > 15) {
      maxScale[0] = 20;
    }
    maxScale[0] = (0.1*maxScale[0]);
  } else {
    maxScale[0] = Math.ceil(maxScale[0]);
    if(maxScale[0] % 3 !== 0 && maxScale[0] % 5 !== 0){
      if(maxScale[0] % 2 !== 0){
        maxScale[0]+=1;
      }
    }
    if(maxScale[0] === 9) maxScale[0]++;
  }
  max = maxScale[0] * Math.pow(10, maxScale[1]);
  return [0, max];
}

var tool = {
  axisHandle: function axisHandle(axis,range,scales,yHandle) {
    var data = axis.data;
    var type = axis.type;
    if(type === 'time'){
      return d3.scalePoint().range(range).domain(data);
    }else if(type === 'points'){
      var numExtend = d3.extent(data);
      var domain = data;
      if(scales){
        scales.yAxisRange = numExtend;
      }
      if(yHandle){
        domain = domainHandle(numExtend);
        return d3.scaleLinear().range(range).domain(domain);
      }
      return d3.scalePoint().range(range).domain(domain);
    }else if(type === 'linear'){
      if(scales){
        scales.yAxisRange = data;
      }
      return d3.scaleLinear().range(range).domain(data);
    }else{
      return d3.scalePoint().range(range).domain(data);
    }
  },
  /**
   * 对象深拷贝
   * 参数：需合并对象1, 需合并对象2
   * 返回合并后对象
   */
  extend : function() {
    var copyIsArray,
        toString = Object.prototype.toString,
        hasOwn = Object.prototype.hasOwnProperty,
    class2type = {
        '[object Boolean]' : 'boolean',
        '[object Number]' : 'number',
        '[object String]' : 'string',
        '[object Function]' : 'function',
        '[object Array]' : 'array',
        '[object Date]' : 'date',
        '[object RegExp]' : 'regExp',
        '[object Object]' : 'object'
    },
    type = function(obj) {
        return obj == null ? String(obj) : class2type[toString.call(obj)] || "object";
    },
    isWindow = function(obj) {
        return obj && typeof obj === "object" && "setInterval" in obj;
    },
    isArray = Array.isArray || function(obj) {
        return type(obj) === "array";
    },
    isPlainObject = function(obj) {
        if (!obj || type(obj) !== "object" || obj.nodeType || isWindow(obj)) {
            return false;
        }
        if (obj.constructor && !hasOwn.call(obj, "constructor")
                && !hasOwn.call(obj.constructor.prototype, "isPrototypeOf")) {
            return false;
        }
        var key;
        for (key in obj) {
        }
        return key === undefined || hasOwn.call(obj, key);
    },
    extend = function(target, options) {
        var deep = true,
          src,copy,clone;
        for (var name in options) {
            src = target[name];
            copy = options[name];
            if (target === copy) { continue; }
            if (deep && copy
                    && (isPlainObject(copy) || (copyIsArray = isArray(copy)))) {
                if (copyIsArray) {
                    copyIsArray = false;
                    clone = src && isArray(src) ? src : [];
                } else {
                    clone = src && isPlainObject(src) ? src : {};
                }
                target[name] = extend(clone, copy);
            } else if (copy !== undefined) {
                target[name] = copy;
            }
        }
        return target;
    };
    return extend;
  }(),
  box: box,
  axisGener: axisGener,
  lineGener: lineGener,
  areaGener: areaGener,
  maskGener: maskGener,
  dotGener: dotGener,
  barGener: barGener
};
module.exports = tool;